@@ -1,7 +1,7 @@
 """Module for the automations and business logic of XKNX."""
 # flake8: noqa
 from .connection_manager import ConnectionManager
-from .connection_state import XknxConnectionState
+from .connection_state import XknxConnectionState, XknxConnectionType
 from .state_updater import StateUpdater
 from .task_registry import Task, TaskRegistry
 from .telegram_queue import TelegramQueue
