@@ -30,15 +30,13 @@ class DPTControlStepCode(DPTBase, ABC):
     APCI_STEPCODEMASK = 0x07
     APCI_MAX_VALUE = APCI_CONTROLMASK | APCI_STEPCODEMASK
 
-    unit = ""
     payload_length = 1
 
     @classmethod
     def _encode(cls, control: bool, step_code: int) -> int:
         """Encode control-bit with step-code."""
         value = 1 if control > 0 else 0
-        value = (value << 3) | (step_code & cls.APCI_STEPCODEMASK)
-        return value
+        return (value << 3) | (step_code & cls.APCI_STEPCODEMASK)
 
     @classmethod
     def _decode(cls, value: int) -> tuple[bool, int]:
@@ -56,9 +54,8 @@ class DPTControlStepCode(DPTBase, ABC):
     @classmethod
     def _test_values(cls, step_code: int) -> bool:
         """Test if input values are valid."""
-        if isinstance(step_code, int):
-            if 0 <= step_code <= cls.APCI_STEPCODEMASK:
-                return True
+        if isinstance(step_code, int) and 0 <= step_code <= cls.APCI_STEPCODEMASK:
+            return True
         return False
 
     @classmethod
@@ -67,7 +64,7 @@ class DPTControlStepCode(DPTBase, ABC):
         # TODO: use Tuple or Named Tuple instead of Dict[str, int] to account for bool control
         if not isinstance(value, dict):
             raise ConversionError(
-                f"Cant serialize {cls.__name__}; invalid value type", value=value
+                f"Can't serialize {cls.__name__}; invalid value type", value=value
             )
 
         try:
@@ -75,12 +72,12 @@ class DPTControlStepCode(DPTBase, ABC):
             step_code = value["step_code"]
         except KeyError:
             raise ConversionError(
-                f"Cant serialize {cls.__name__}; invalid keys", value=value
+                f"Can't serialize {cls.__name__}; invalid keys", value=value
             )
 
         if not cls._test_values(step_code):
             raise ConversionError(
-                f"Cant serialize {cls.__name__}; invalid values", value=value
+                f"Can't serialize {cls.__name__}; invalid values", value=value
             )
 
         return (cls._encode(control, step_code),)
@@ -89,7 +86,7 @@ class DPTControlStepCode(DPTBase, ABC):
     def from_knx(cls, raw: tuple[int, ...]) -> Any:
         """Parse/deserialize from KNX/IP raw data."""
         if not isinstance(raw, tuple) or not cls._test_boundaries(raw[0]):
-            raise ConversionError(f"Cant parse {cls.__name__}", raw=raw)
+            raise ConversionError(f"Can't parse {cls.__name__}", raw=raw)
 
         control, step_code = cls._decode(raw[0])
 
@@ -97,10 +94,9 @@ class DPTControlStepCode(DPTBase, ABC):
 
 
 class DPTControlStepwise(DPTControlStepCode):
-    """Abstraction for KNX DPT 3.xxx in stepwise mode with conversion to an incement value."""
+    """Abstraction for KNX DPT 3.xxx in stepwise mode with conversion to an increment value."""
 
     dpt_main_number = 3
-    dpt_sub_number: int | None = None
     value_type = "stepwise"
     unit = "%"
 
@@ -143,7 +139,7 @@ class DPTControlStepwise(DPTControlStepCode):
     def to_knx(cls, value: int | dict[str, int]) -> tuple[int]:
         """Serialize to KNX/IP raw data."""
         if not isinstance(value, int):
-            raise ConversionError(f"Cant serialize {cls.__name__}", value=value)
+            raise ConversionError(f"Can't serialize {cls.__name__}", value=value)
 
         return super().to_knx(cls._from_increment(value))
 
@@ -184,7 +180,6 @@ class DPTControlStartStop(DPTControlStepCode):
     """Abstraction for KNX DPT 3.xxx in start/stop mode."""
 
     value_type = "startstop"
-    unit = ""
 
     class Direction(TitleEnum):
         """Enum for indicating the direction."""
@@ -208,7 +203,7 @@ class DPTControlStartStop(DPTControlStepCode):
             control = 0
             step_code = 0
         else:
-            raise ConversionError(f"Cant serialize {cls.__name__}", value=value)
+            raise ConversionError(f"Can't serialize {cls.__name__}", value=value)
 
         values = {"control": control, "step_code": step_code}
         return super().to_knx(values)
