@@ -29,7 +29,6 @@ class DPT4ByteFloat(DPTNumeric):
     dpt_main_number = 14
     dpt_sub_number: int | None = None
     value_type = "4byte_float"
-    unit = ""
     payload_length = 4
 
     value_min = float("-inf")
@@ -45,7 +44,7 @@ class DPT4ByteFloat(DPTNumeric):
         except struct.error:
             raise ConversionError(f"Could not parse {cls.__name__}", raw=raw)
         try:
-            # round to 7 digit precicion independent of exponent - same value as ETS 5.7 group monitor
+            # round to 7 digit precision independent of exponent - same value as ETS 5.7 group monitor
             return round(raw_float, 7 - ceil(log10(abs(raw_float))))
         except (ValueError, OverflowError):
             # account for 0 and special values
@@ -60,7 +59,7 @@ class DPT4ByteFloat(DPTNumeric):
             knx_value = float(value)
             return tuple(struct.pack(">f", knx_value))
         except (ValueError, struct.error):
-            raise ConversionError(f"Could not serialize {cls.__name__}", vlaue=value)
+            raise ConversionError(f"Could not serialize {cls.__name__}", value=value)
 
 
 class DPTAcceleration(DPT4ByteFloat):
@@ -114,7 +113,6 @@ class DPTAmplitude(DPT4ByteFloat):
     dpt_main_number = 14
     dpt_sub_number = 5
     value_type = "amplitude"
-    unit = ""
 
 
 class DPTAngleRad(DPT4ByteFloat):
@@ -241,6 +239,7 @@ class DPTElectricCurrent(DPT4ByteFloat):
     dpt_sub_number = 19
     value_type = "electric_current"
     unit = "A"
+    ha_device_class = "current"
 
 
 class DPTElectricCurrentDensity(DPT4ByteFloat):
@@ -367,6 +366,7 @@ class DPTFrequency(DPT4ByteFloat):
     dpt_sub_number = 33
     value_type = "frequency"
     unit = "Hz"
+    ha_device_class = "frequency"
 
 
 class DPTAngularFrequency(DPT4ByteFloat):
@@ -421,6 +421,7 @@ class DPTLength(DPT4ByteFloat):
     dpt_sub_number = 39
     value_type = "length"
     unit = "m"
+    ha_device_class = "distance"
 
 
 class DPTLightQuantity(DPT4ByteFloat):
@@ -439,7 +440,6 @@ class DPTLuminance(DPT4ByteFloat):
     dpt_sub_number = 41
     value_type = "luminance"
     unit = "cd/m²"
-    ha_device_class = "illuminance"
 
 
 class DPTLuminousFlux(DPT4ByteFloat):
@@ -449,7 +449,6 @@ class DPTLuminousFlux(DPT4ByteFloat):
     dpt_sub_number = 42
     value_type = "luminous_flux"
     unit = "lm"
-    ha_device_class = "illuminance"
 
 
 class DPTLuminousIntensity(DPT4ByteFloat):
@@ -459,7 +458,6 @@ class DPTLuminousIntensity(DPT4ByteFloat):
     dpt_sub_number = 43
     value_type = "luminous_intensity"
     unit = "cd"
-    ha_device_class = "illuminance"
 
 
 class DPTMagneticFieldStrength(DPT4ByteFloat):
@@ -532,6 +530,7 @@ class DPTMass(DPT4ByteFloat):
     dpt_sub_number = 51
     value_type = "mass"
     unit = "kg"
+    ha_device_class = "weight"
 
 
 class DPTMassFlux(DPT4ByteFloat):
@@ -586,7 +585,6 @@ class DPTPowerFactor(DPT4ByteFloat):
     dpt_main_number = 14
     dpt_sub_number = 57
     value_type = "powerfactor"
-    unit = "cosΦ"
     ha_device_class = "power_factor"
 
 
@@ -624,7 +622,7 @@ class DPTResistivity(DPT4ByteFloat):
     dpt_main_number = 14
     dpt_sub_number = 61
     value_type = "resistivity"
-    unit = "Ω m"
+    unit = "Ωm"
 
 
 class DPTSelfInductance(DPT4ByteFloat):
@@ -661,6 +659,7 @@ class DPTSpeed(DPT4ByteFloat):
     dpt_sub_number = 65
     value_type = "speed"
     unit = "m/s"
+    ha_device_class = "speed"
 
 
 class DPTStress(DPT4ByteFloat):
@@ -750,7 +749,7 @@ class DPTTorque(DPT4ByteFloat):
     dpt_main_number = 14
     dpt_sub_number = 75
     value_type = "torque"
-    unit = "N m"
+    unit = "Nm"
 
 
 class DPTVolume(DPT4ByteFloat):
@@ -787,3 +786,13 @@ class DPTWork(DPT4ByteFloat):
     dpt_sub_number = 79
     value_type = "work"
     unit = "J"
+
+
+class DPTApparentPower(DPT4ByteFloat):
+    """DPT 14.080 DPT_Value_Apparent_Power."""
+
+    dpt_main_number = 14
+    dpt_sub_number = 80
+    value_type = "apparent_power"
+    unit = "VA"
+    ha_device_class = "apparent_power"
