@@ -40,10 +40,9 @@ class DPTDateTime(DPTBase):
             if weekday == 0:
                 # struct_time has no concept of "no/any day"
                 weekday_invalid = True
-            else:
+            elif weekday == 7:
                 # in knx Sunday is 7; in strftime %w its 0
-                if weekday == 7:
-                    weekday = 0
+                weekday = 0
             # string conversion used for catching exceptions and inferring invalid data
             _time_strings = []
             _time_formats = []
