@@ -5,8 +5,10 @@ from .exception import (
     ConfirmationError,
     ConversionError,
     CouldNotParseAddress,
+    CouldNotParseCEMI,
     CouldNotParseKNXIP,
     CouldNotParseTelegram,
+    DataSecureError,
     DeviceIllegalValue,
     IncompleteKNXIPFrame,
     InvalidSecureConfiguration,
@@ -25,8 +27,10 @@ __all__ = [
     "ConfirmationError",
     "ConversionError",
     "CouldNotParseAddress",
+    "CouldNotParseCEMI",
     "CouldNotParseKNXIP",
     "CouldNotParseTelegram",
+    "DataSecureError",
     "DeviceIllegalValue",
     "ManagementConnectionError",
     "ManagementConnectionRefused",
