@@ -7,7 +7,7 @@ from __future__ import annotations
 
 import asyncio
 import logging
-from typing import Callable, cast
+from typing import Callable
 
 from xknx.exceptions import CommunicationError, CouldNotParseKNXIP, IncompleteKNXIPFrame
 from xknx.knxip import HPAI, HostProtocol, KNXIPFrame
@@ -35,6 +35,19 @@ class TCPTransport(KNXIPTransport):
             self.data_received_callback = data_received_callback
             self.connection_lost_callback = connection_lost_callback
 
+            # Workaround for issue of TCP Transport in ProactorEventLoop in py3.10 and py3.11
+            # on Windows returning bytearray instead of bytes which lead to errors in
+            # cryptography (eg. X25519PublicKey.from_public_bytes() in IP Secure handshake)
+            # https://github.com/python/cpython/issues/99941
+            try:
+                if isinstance(asyncio.get_event_loop(), asyncio.ProactorEventLoop):  # type: ignore[attr-defined]
+                    self.data_received_callback = lambda data: data_received_callback(
+                        bytes(data)
+                    )
+            except AttributeError:
+                # asyncio.ProactorEventLoop is only available on Windows
+                pass
+
         def connection_made(self, transport: asyncio.BaseTransport) -> None:
             """Assign transport. Callback after udp connection was made."""
             self.transport = transport
@@ -103,13 +116,11 @@ class TCPTransport(KNXIPTransport):
             connection_lost_callback=self._connection_lost,
         )
         loop = asyncio.get_running_loop()
-        (transport, _) = await loop.create_connection(
+        (self.transport, _) = await loop.create_connection(
             lambda: tcp_transport_factory,
             host=self.remote_hpai.ip_addr,
             port=self.remote_hpai.port,
         )
-        # TODO: typing - remove cast - loop.create_connection should return a asyncio.Transport
-        self.transport = cast(asyncio.Transport, transport)
 
     def _connection_lost(self) -> None:
         """Call assigned callback. Callback for connection lost."""
